/**
 * @description All parent nodes will be resized and their children will be remove from the graph.
 */
function collapse_all () {
    var busy = false, task = "collapse_group", group_iterator = new IteratorGraph(_nodes.groups);
    // Préparation des tâches, de l'itérateur, et d'une variable qui permettra de "bloqué" le code dans l'interval.

    if (group_iterator.size() > 0) {
        $("#collapsing").removeClass("hidden"); // Affichage d'un message informatif

        group_iterator.beforeFirst(); // Déplacement du curseur de l'iterateur.
        var interval = setInterval(function () {
            // Dans cette insertion de code, le code est bloquant ce qui signifie que pour cette partie de code,
            // tous ce fait de haut en bas et si une commande dure dans le temps, la suite s'en trouve bloqué.
            
            if (!busy) { // Contrôle de tâches (occupé ne rajoute rien, libre fait la prochaine tâche).
                busy = !busy; // Inversion libre/occupé

                switch (task) {
                    case "collapse_group":
                        if (group_iterator.hasNext()) { // Si next
                            if (!group_iterator.next().hasClass("collapse_group")) { // Placé sur next et demande s'il est pas déjà collapse
                                collapse_element(group_iterator.current()); // Si pas collapse on le collapse
                            }
                        } else {
                            task = "finish"; // Si pas de next bascule sur tâche 'finish'
                        }
                        break;
                    case "finish":
                        $("#collapsing").addClass("hidden"); // Enlèvement du message informatif
                        clearInterval(interval); // Termine l'interval.
                        break;
                }
                
                busy = !busy; // Quand la tâche est finis on n'est plus occupé.
            }
        }, 100); // Interval toute les 100 millisecondes.
    }
}
/**
 * @description Collapse a specific element.
 * @param {object} node
 */
function collapse_element (node) {
    // Cette fonction si faîte sur demande express de l'utilisateur est bloquante pour l'UI,
    // si faîte par le biais de la fonction collapse_all elle n'est pas bloquante pour l'UI.
    var node_iterator = new IteratorGraph(node.children());

    node_iterator.forEach(function (child) {
        if (child.selected()) child.unselect();

        var incomersEdges = child.incomers("edge"), // Les edges dont le child est target
            outgoersEdges = child.outgoers("edge"); // Les edges dont le child est source

        moveEdges(incomersEdges, child.id(), edge_a.data.target, {target: node.id()}); // Déplace la target des edges de child vers parent
        moveEdges(outgoersEdges, child.id(), edge_a.data.source, {source: node.id()}); // Déplace la source des edges de child vers parent

        removeAllClasses(child);
        child.remove() // ; pas obligé pour la dernière instruction d'une fonction en JavaScript - Pas bien je viens juste de le remarqué.
    });

    /* Adding collapse class to the node group. */
    node.addClass("collapse_group");
    node.removeClass("expand_group");

    if (node.selected()) {
        majGroup(node, true);
    } else {
        majNodeColor(node);
    }
    /**
     * @description Moving the given edges and storing their original location.
     * @param {object} edges A collection of cytoscape edges.
     * @param {string} originalId
     * @param {string} linkWay
     * @param {{source:string}|{target:string}} newLocation
     */
    function moveEdges (edges, originalId, linkWay, newLocation) {
        for (var e = 0, edge = edges[e]; e < edges.length; e++, edge = edges[e]) {
            if (linkWay === edge_a.data.source) edge.data(edge_a.data.realSource, originalId);
            else                                edge.data(edge_a.data.realTarget, originalId);

            edge.move(newLocation);
        }
    }
}
/**
 * @description Expand all group.
 */
function expand_all () {
    var busy = false, task = "expand_group", group_iterator = new IteratorGraph(_nodes.groups);

    if (group_iterator.size() > 0) {
        $("#expanding").removeClass("hidden");

        group_iterator.beforeFirst();
        var interval = setInterval(function () {
            if (!busy) {
                busy = !busy;

                switch (task) {
                    case "expand_group":
                        if (group_iterator.hasNext()) {
                            if (!group_iterator.next().hasClass("expand_group")) {
                                expand_element(group_iterator.current());
                            }
                        } else {
                            task = "finish";
                        }
                        break;
                    case "finish":
                        $("#expanding").addClass("hidden");
                        clearInterval(interval);
                        break;
                }

                busy = !busy;
            }
        }, 100);
    }
}
/**
 * @description Expand a specific element.
 * @param {object} node A Cytoscape node.
 */
function expand_element (node) {
    /* Resizing the node group. */
    var json_node = node.data(node_a.data.hiddenProperties)[node_p.me];

    var node_iterator = new IteratorGraph(json_node.getProp(node_p.cytoscapeNodes));

    node_iterator.forEach(function (child) {
        child.restore();

        /* Moving the edges to their original target/source. */
        node.incomers("edge[" + edge_a.data.realTarget + "='" + child.id() + "']").forEach(function (edge) { edge.move({target : child.id()}); });
        node.outgoers("edge[" + edge_a.data.realSource + "='" + child.id() + "']").forEach(function (edge) { edge.move({source : child.id()}); });

        majNodeColor(child);
    });

    /* Removing collapse class to the node group. */
    node.addClass("expand_group");
    node.removeClass("collapse_group");

    /* Don't forget to select the child nodes if the group node is selected. */
    if (node.selected()) {
        node.children().select();
    } else {
        majNodeColor(node);
    }
}